#include <stm32f0xx.h>
#include <gpio.hpp>

using namespace stm32f072xb;

/**
 * Read section 9 (p.165) for mapping GPIOs to EXTI lines
 * Read section 11 (p.209) for EXTI configuration
 * See p.936 for external interrupt example
 * See Nucleo64 documentation to find pin corresponding to user button (PIN C13)
 * My savior : https://vivonomicon.com/2018/04/28/bare-metal-stm32-programming-part-4-intro-to-hardware-interrupts/
**/

extern "C" void EXTI4_15_IRQHandler()
{
	SET_BIT(EXTI->PR, EXTI_PR_PIF13); // Clear interrupt register (p.171)

	if (!READ_BIT(GPIOC->IDR, GPIO_IDR_13)) // Read PIN C13
	{
		SET_BIT(GPIOA->ODR, GPIO_ODR_5); // Turn on LED (p.159)
	}
	else
	{
		CLEAR_BIT(GPIOA->ODR, GPIO_ODR_5); // Turn off LED (p.159)
	}
}

void setupGpio()
{
	/* Enabling GPIOs is not necessary for external interrupts */
	SET_BIT(RCC->AHBENR, RCC_AHBENR_GPIOAEN | RCC_AHBENR_GPIOCEN); // Enable GPIOA and GPIOC (p. 120)
}

void setupLed()
{
	CLEAR_BIT(GPIOA->MODER, GPIO_MODER_MODER5);
	SET_BIT(GPIOA->MODER, GPIO_MODER_MODER5_0); // Output mode (p.157)
	CLEAR_BIT(GPIOA->OTYPER, GPIO_OTYPER_OT_5); // Push-pull output (p.157)
	CLEAR_BIT(GPIOA->ODR, GPIO_ODR_5);			// Turn off LED, p.159
}

void systemConfig()
{ // Enable clock for System Config and Comp (p.121)
	SET_BIT(RCC->APB2ENR, RCC_APB2ENR_SYSCFGCOMPEN);

	// Connect button pin to EXTI line (p.171)
	CLEAR_BIT(SYSCFG->EXTICR[3], SYSCFG_EXTICR4_EXTI13_Msk);
	SET_BIT(SYSCFG->EXTICR[3], SYSCFG_EXTICR4_EXTI13_PC);
}

void setupInterrupts()
{
	constexpr uint16_t interruptPriority = 0;

	SET_BIT(EXTI->IMR, EXTI_IMR_IM13);	 // Enable external interrupts coming from line 13
	SET_BIT(EXTI->RTSR, EXTI_RTSR_RT13); // Rising trigger enabled for input line 13
	SET_BIT(EXTI->FTSR, EXTI_FTSR_FT13); // Falling trigger ENABLED for input line 13

	NVIC_EnableIRQ(EXTI4_15_IRQn);						// Enable interrupts for EXTI lines 4 to 15 (p. 936)
	NVIC_SetPriority(EXTI4_15_IRQn, interruptPriority); // Sets interrupts' priority to the highest (0)
}

int main()
{
	setupGpio();
	setupLed();
	systemConfig();
	setupInterrupts();

	while (true)
		;
}
